
from api_helpers.message_handler import MessageHandler
from api_helpers.connection_manager import ConnectionManager

from geometry_msgs.msg import Twist
# robobin/api_node.py
import rclpy
from rclpy.node import Node

class ApiNode(Node):
    def __init__(self):
        super().__init__('api_node')
        self.get_logger().info("ApiNode has been started.")

        self.publisher_topics = {
            "cmd_vel": self.create_publisher(Twist, '/cmd_vel', 10),
            "nav_send": self.create_publisher(Twist, '/nav_send', 10), 
            "map": self.create_publisher(Twist, '/map', 10) 
        }
        subscriber_topics = {
            "location": "/location"
        }
        self.message_handler = MessageHandler(self)
        self.connection_manager = ConnectionManager(self)

        self.connection_manager.start()

    def handle_client_connection(self, client_socket):
        """Handles incoming TCP client connections."""
        try:
            while True:
                data = client_socket.recv(1024).decode()
                if not data:
                    break
                topic,message = self.message_handler.handle_message(client_socket, data)
                if topic is not None:
                    self.publish_to_topic(topic, message)
        finally:
            client_socket.close()
            self.get_logger().info("Client disconnected.")
    def publish_to_topic(self, topic, message):
        """Publishes the message to the specified topic."""
        if topic in self.publisher_topics:
            publisher = self.publisher_topics[topic]

            # Check if the topic is 'cmd_vel' and format the message as a Twist message
            if topic == "cmd_vel" and isinstance(message, tuple):
                linear_x, angular_z = message
                twist_msg = Twist()
                twist_msg.linear.x = linear_x
                twist_msg.linear.y = 0.0
                twist_msg.linear.z = 0.0
                twist_msg.angular.x = 0.0
                twist_msg.angular.y = 0.0
                twist_msg.angular.z = angular_z

                publisher.publish(twist_msg)
                self.get_logger().info(f"Published to {topic}: linear_x={linear_x}, angular_z={angular_z}")
            else:
                self.get_logger().warning(f"Unhandled message type for topic: {topic}")
        else:
            self.get_logger().warning(f"Unknown topic: {topic}")

    def shutdown(self):
        """Stops the connection manager."""
        self.connection_manager.stop()

def main(args=None):
    rclpy.init(args=args)
    node = ApiNode()
    try:
        rclpy.spin(node)
    except KeyboardInterrupt:
        node.shutdown()
    finally:
        node.destroy_node()
        rclpy.shutdown()

if __name__ == '__main__':
    main()
